/*
 * FTMEXT SDK example
 * Copyright (C) 2004 by IRCAM-Centre Georges Pompidou, Paris, France.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * See file COPYING.LIB
 for further informations on licensing terms.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
#include "ftmlib.h"

typedef struct
{
  ftmext_t o;
  double period;
  double ksr;
  int index;
  int side;
} pingpong_t;

/************************************************
 *
 *  dsp
 *
 */
FTMEXT_DSP_PERFORM(pingpong)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  float *in = FTMEXT_GET_DSP_INPUT(0);
  int n_tick = FTMEXT_GET_DSP_SIZE();
  int period = (int)(self->period * self->ksr);
  int side = self->side;
  int index = self->index;
  int left = period - index;
  int onset = 0;
  
  if(left < 0)
  {
    left = 0;
    index = period;
  }
  
  while(onset < n_tick)
  {
    float *out_1 = FTMEXT_GET_DSP_OUTPUT(side);
    float *out_0 = FTMEXT_GET_DSP_OUTPUT(1 - side);
    int i;
    
    if(left > n_tick - onset)
      left = n_tick - onset;
    
    for(i=onset; i<onset+left; i++)
    {
      out_1[i] = in[i];
      out_0[i] = 0.0;
    }
    
    index += left;
    onset += left;
    
    if(index == period)
    {
      side = 1 - side;
      index = 0;
      ftmext_alarm_set((ftmext_t *)self, 0.0);
    }
    
    left = period - index;
  }
  
  self->index = index;
  self->side = side;
  
  FTMEXT_DSP_PERFORM_RETURN;
}

FTMEXT_DSP_INIT(pingpong)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  double sr = FTMEXT_GET_DSP_SR();
  int n_tick = FTMEXT_GET_DSP_SIZE();
  
  self->ksr = 0.001 * sr;
  
  FTMEXT_DSP_INIT_RETURN;
}

/************************************************
 *
 *  methods
 *
 */
FTMEXT_METHOD_VOID(pingpong, bang)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  
  ftmext_outlet_bang((ftmext_t *)self, 2);
  
  FTMEXT_METHOD_RETURN;
}

FTMEXT_METHOD_NUMBER(pingpong, period)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  double period = FTMEXT_GET_FLOAT();
  
  if(period < 1.0)
    period = 1.0;
  
  self->period = period;
  
  FTMEXT_METHOD_RETURN;
}


FTMEXT_ALARM(pingpong)
{
  pingpong_bang(FTMEXT_GET_EXT());
  
  FTMEXT_ALARM_RETURN;
}


FTMEXT_INIT(pingpong)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  
  self->period = 200.0;
  self->ksr = 1.0;
  self->index = 0;
  self->side = 0;
  
  FTMEXT_INIT_RETURN;
}

FTMEXT_DELETE(pingpong)
{
  pingpong_t *self = (pingpong_t *)FTMEXT_GET_EXT();
  
  FTMEXT_DELETE_RETURN;
}

/* main function for Max/MSP */
FTMEXT_CLASS_NAMED(fff, pingpong, pingpong_t, "fff.pingpong~")
{
  FTMEXT_CLASS_DESCR(_prefix_ extclass, "FTM audio external example", "Example of an FTM audio external");
  FTMEXT_CLASS_DSP(pingpong);
  
  FTMEXT_CLASS_ARGUMENT_NUMBER(pingpong, 0, period, 200.0, "period");
  FTMEXT_CLASS_ATTRIBUTE_NUMBER(pingpong, period, FTMEXT_ATTRIBUTE_DYNAMIC, "period");
  
  FTMEXT_CLASS_INLET_SIGNAL(pingpong, 0, "input signal");
  FTMEXT_CLASS_OUTLET_SIGNAL(pingpong, 0, "output signal low");
  FTMEXT_CLASS_OUTLET_SIGNAL(pingpong, 1, "output signal high");
  FTMEXT_CLASS_OUTLET_BANG(pingpong, 2, "bang for each transition");
  
  FTMEXT_CLASS_ALARM(pingpong, "bang at transition");
  
  FTMEXT_CLASS_RETURN;
}

